﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MotorBoat
{
    internal abstract class AbstractMap : IEquatable<AbstractMap>
    {
        private IDrawningObject _drawningObject = null;
        protected int[,] _map = null;
        protected int _width;
        protected int _height;
        protected float _size_x;
        protected float _size_y;
        protected readonly Random _random = new();
        protected readonly int _freeRoad = 0;
        protected readonly int _barrier = 1;
        public Bitmap CreateMap(int width, int height, IDrawningObject drawningObject)
        {
            _width = width;
            _height = height;
            _drawningObject = drawningObject;
            GenerateMap();
            while (!SetObjectOnMap())
            {
                GenerateMap();
            }
            return DrawMapWithObject();
        }
        public Bitmap MoveObject(Direction direction)
        {
            // TODO проверка, что объект может переместится в требуемом направлении
            (float leftX, float topY, float rightX, float bottomY) = _drawningObject.GetCurrentPosition();
            bool can = true;
            //координаты в системе массива
            int topYinS = Convert.ToInt32((topY) / _size_y);
            int rightXinS = Convert.ToInt32((rightX) / _size_x);
            int leftXinS = Convert.ToInt32((leftX) / _size_x);
            int bottomYinS = Convert.ToInt32(bottomY / _size_y);

            int stepinS = 0; 

            switch (direction)
            {
                case Direction.Up:
                    stepinS = Convert.ToInt32((topY - _drawningObject.Step) / _size_y);
                    if (stepinS < 0) stepinS = 0;
                    if (stepinS != topYinS)
                    {
                        for (int j = topYinS; j > stepinS; j--)
                        {
                            for (int i = leftXinS; i < rightXinS; i++)
                            {
                                if (_map[i, j] == _barrier)
                                {
                                    can = false;
                                }
                            }
                        }
                        if (!can)
                        {
                            return DrawMapWithObject();
                        }
                        _drawningObject.MoveObject(direction);
                    }
                    return DrawMapWithObject();

                case Direction.Down:
                    stepinS = Convert.ToInt32((bottomY + _drawningObject.Step) / _size_y);
                    if (stepinS > _height) stepinS = _height;
                    if (stepinS != topYinS)
                    {
                        for (int j = topYinS; j < stepinS; j++)
                        {
                            for (int i = leftXinS; i < rightXinS; i++)
                            {
                                if (_map[i, j] == _barrier)
                                {
                                    can = false;
                                }
                            }
                        }
                        if (!can)
                        {
                            return DrawMapWithObject();
                        }
                        _drawningObject.MoveObject(direction);
                    }
                    return DrawMapWithObject();

                case Direction.Left:
                    stepinS = Convert.ToInt32((leftX - _drawningObject.Step) / _size_x);
                    if (stepinS < 0) stepinS = 0;
                    if (stepinS != leftXinS)
                    {
                        for (int j = topYinS; j < bottomYinS; j++)
                        {
                            for (int i = leftXinS; i > stepinS; i--)
                            {
                                if (_map[i, j] == _barrier)
                                {
                                    can = false;
                                }
                            }
                        }
                        if (!can)
                        {
                            return DrawMapWithObject();
                        }
                        _drawningObject.MoveObject(direction);
                    }
                    return DrawMapWithObject();

                case Direction.Right:
                    stepinS = Convert.ToInt32((rightX + _drawningObject.Step) / _size_x);
                    if (stepinS > _width) stepinS = _width;
                    if (stepinS != leftXinS)
                    {
                        for (int j = topYinS; j < bottomYinS; j++)
                        {
                            for (int i = rightXinS; i < stepinS; i++)
                            {
                                if (_map[i, j] == _barrier)
                                {
                                    can = false;
                                }
                            }
                        }
                        if (!can)
                        {
                            return DrawMapWithObject();
                        }
                        _drawningObject.MoveObject(direction);
                    }
                    return DrawMapWithObject();
            }
            return DrawMapWithObject();
        }
        private bool SetObjectOnMap()
        {
           if (_drawningObject == null || _map == null)
            {
                return false;
            }
            int x = _random.Next(0, 10);
            int y = _random.Next(0, 10);
            _drawningObject.SetObject(x, y, _width, _height);
            // TODO првоерка, что объект не "накладывается" на закрытые участки
            (float leftX, float topY, float rightX, float bottomY) = _drawningObject.GetCurrentPosition();
            //координаты лодки в "клетках" массива карты
            int topYinS = Convert.ToInt32(topY / _size_y);
            int rightXinS = Convert.ToInt32(rightX / _size_x);
            int leftXinS = Convert.ToInt32(leftX / _size_x);
            int bottomYinS = Convert.ToInt32(bottomY / _size_y);
            
            if (leftXinS < 0 || bottomYinS > _height || topYinS<0 || rightX > _width) {  return false; }

            for (int j = topYinS; j <= bottomYinS; j++)
            {
                for (int i = leftXinS; i <= rightXinS; i++)
                {
                    if (_map[i, j] == _barrier)
                    {
                        return false;
                    }
                }
            }

            return true;
        }

        public bool Equals(AbstractMap? other)
        {
            if (other == null)
            {
                return false;
            }
            var otherMap = other as AbstractMap;
            if (otherMap == null)
            {
                return false;
            }
            if (_width != otherMap._width)
            {
                return false;
            }
            if (_height != otherMap._height)
            {
                return false;
            }
            if (_size_x != otherMap._size_x)
            {
                return false;
            }
            if (_size_y != otherMap._size_y)
            {
                return false;
            }
            for (int i = 0; i < _map.GetLength(0); i++)
            {
                for (int j = 0; j < _map.GetLength(1); j++)
                {
                    if (_map[i, j] != otherMap._map[i, j])
                    {
                        return false;
                    }
                }
            }
            return true;
        }

        private Bitmap DrawMapWithObject()
        {
            Bitmap bmp = new(_width, _height);
            if (_drawningObject == null || _map == null)
            {
                return bmp;
            }
            Graphics gr = Graphics.FromImage(bmp);
            for (int i = 0; i < _map.GetLength(0); ++i)
            {
                for (int j = 0; j < _map.GetLength(1); ++j)
                {
                    if (_map[i, j] == _freeRoad)
                    {
                        DrawRoadPart(gr, i, j);
                    }
                    else if (_map[i, j] == _barrier)
                    {
                        DrawBarrierPart(gr, i, j);
                    }
                }
            }
            _drawningObject.DrawningObject(gr);
            return bmp;
        }

        protected abstract void GenerateMap();
        protected abstract void DrawRoadPart(Graphics g, int i, int j);
        protected abstract void DrawBarrierPart(Graphics g, int i, int j);
    }
}
