﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MotorBoat
{
    public class DrawningBoat
    {
        public EntityBoat Boat { get; protected set; }

        protected float _startPosX;
        protected float _startPosY;

        private int? _pictureWidth = null;
        private int? _pictureHeight = null;

        private readonly int _boatWidth = 70;
        private readonly int _boatHeight = 40;
        public DrawningBoat(int speed, float weight, Color bodyColor)
        {
            Boat = new EntityBoat(speed, weight, bodyColor);
        }
        protected DrawningBoat(int speed, float weight, Color bodyColor, int boatWidth, int boatHeight) :
            this(speed, weight, bodyColor)
        {
            _boatWidth = boatWidth;
            _boatHeight = boatHeight;
        }
        public void SetPosition(int x, int y, int width, int height)
        {
            // TODO checks
            _startPosX = x;
            _startPosY = y;
            _pictureWidth = width;
            _pictureHeight = height;
            if (_startPosX + _boatWidth > _pictureWidth) { _startPosX = 10; }
            if (_startPosY - _boatHeight < 0) { _startPosY = _boatHeight + 10; }
            if (_startPosY + _boatHeight > _pictureHeight) { _startPosY -= _boatHeight; }
        }
        public void MoveTransport(Direction direction)
        {
            if (!_pictureWidth.HasValue || !_pictureHeight.HasValue)
            {
                return;
            }
            switch (direction)
            {
                // вправо
                case Direction.Right:
                    if (_startPosX + _boatWidth + Boat.Step < _pictureWidth)
                    {
                        _startPosX += Boat.Step;
                    }
                    break;
                //влево
                case Direction.Left:
                    if (_startPosX - Boat.Step > 0)
                    {
                        _startPosX -= Boat.Step;
                    }
                    break;
                //вверх
                case Direction.Up:
                    if (_startPosY - _boatHeight - Boat.Step > 0)
                    {
                        _startPosY -= Boat.Step;
                    }
                    break;
                //вниз
                case Direction.Down:
                    if (_startPosY + _boatHeight + Boat.Step < _pictureHeight)
                    {
                        _startPosY += Boat.Step;
                    }
                    break;
            }
        }
        public virtual void DrawTransport(Graphics g)
        {
            if (_startPosX < 0 || _startPosY < 0
                || !_pictureHeight.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }
            Pen pen = new Pen(Color.Black);
            //границы лодки
            Point[] points = new Point[5]
            {
                new Point(Convert.ToInt32(_startPosX),Convert.ToInt32(_startPosY)),
            new Point(Convert.ToInt32(_startPosX + 50),Convert.ToInt32(_startPosY)),
            new Point(Convert.ToInt32(_startPosX + 70),Convert.ToInt32(_startPosY - 20)),
            new Point(Convert.ToInt32(_startPosX + 50),Convert.ToInt32(_startPosY - 40)),
            new Point(Convert.ToInt32(_startPosX),Convert.ToInt32(_startPosY-40)),
            };

            Brush brBody = new SolidBrush(Boat?.BodyColor ?? Color.Black);
            g.FillPolygon(brBody, points);
            g.DrawPolygon(pen, points);
                        
            Brush brYellow = new SolidBrush(Color.Yellow);
            g.FillEllipse(brYellow, _startPosX + 5, _startPosY - 30, 50, 20);
            g.DrawEllipse(pen, _startPosX + 5, _startPosY - 30, 50, 20);

        }
        public void ChangeBorders(int width, int height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_pictureWidth <= _boatWidth || _pictureHeight <= _boatHeight)
            {
                _pictureWidth = null;
                _pictureHeight = null;
                return;
            }
            if (_startPosX + _boatWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth.Value - _boatWidth;
            }
            if (_startPosY + _boatHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight.Value - _boatHeight;
            }
        }
        public (float Left, float Right, float Top, float Bottom) GetCurrentPosition()
        {
            return (_startPosX, _startPosY, _startPosX + _boatWidth, _startPosY + _boatHeight);
        }
        public void SetBodyColor(Color color)
        {
            (Boat as EntityBoat).setColor(color);
        }
    }
}
