﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MotorBoat
{
    internal class DrawningMotorBoat : DrawningBoat
    {
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес автомобиля</param>
        /// <param name="bodyColor">Цвет кузова</param>
        /// <param name="dopColor">Дополнительный цвет</param>
        /// <param name="bodyKit">Признак наличия обвеса</param>
        /// <param name="wing">Признак наличия антикрыла</param>
        /// <param name="sportLine">Признак наличия гоночной полосы</param>
        public DrawningMotorBoat(int speed, float weight, Color bodyColor, Color dopColor, bool bodyKit, bool wing, bool sportLine) :
            base(speed, weight, bodyColor, 110, 60)
        {
            Boat = new EntityMotorBoat(speed, weight, bodyColor, dopColor, bodyKit, wing, sportLine);
        }
        public override void DrawTransport(Graphics g)
        {
            if (Boat is not EntityMotorBoat motorBoat)
            {
                return;
            }
            Point[] points;
            Pen pen = new(Color.Black);
            Brush dopBrush = new SolidBrush(motorBoat.DopColor);
                       
            g.FillEllipse(new SolidBrush(Color.Black), _startPosX - 17, _startPosY - 24, 14, 5);
            g.FillEllipse(new SolidBrush(Color.Black), _startPosX - 17, _startPosY - 21, 14, 5);

            g.FillRectangle(dopBrush, _startPosX - 10, _startPosY - 30, 10, 20);
            g.DrawRectangle(pen, _startPosX - 10, _startPosY - 30, 10, 20);

            if (motorBoat.BodyKit)
            {
                points = new Point[]
                {
                    new Point(Convert.ToInt32(_startPosX),Convert.ToInt32(_startPosY-50)),
                    new Point(Convert.ToInt32(_startPosX),Convert.ToInt32(_startPosY+10)),
                    new Point(Convert.ToInt32(_startPosX+50),Convert.ToInt32(_startPosY)),
                    new Point(Convert.ToInt32(_startPosX+50),Convert.ToInt32(_startPosY-40)),
                };
                g.FillPolygon(dopBrush, points);
                g.DrawPolygon(pen, points);
            }

            if (motorBoat.Wing)
            {
                points = new Point[]
               {
                    new Point(Convert.ToInt32(_startPosX)+50,Convert.ToInt32(_startPosY-40)),
                    new Point(Convert.ToInt32(_startPosX)+80,Convert.ToInt32(_startPosY-20)),
                    new Point(Convert.ToInt32(_startPosX+50),Convert.ToInt32(_startPosY)),
               };
                g.FillPolygon(dopBrush, points);
                g.DrawPolygon(pen, points);
            }

            base.DrawTransport(g);
            points = new Point[]
               {
                    new Point(Convert.ToInt32(_startPosX)+50,Convert.ToInt32(_startPosY-34)),
                    new Point(Convert.ToInt32(_startPosX)+57,Convert.ToInt32(_startPosY-20)),
                    new Point(Convert.ToInt32(_startPosX+50),Convert.ToInt32(_startPosY-6)),
               };
            g.FillPolygon(new SolidBrush(Color.LightBlue), points);
            g.DrawPolygon(pen, points);

            if (motorBoat.SportLine)
            {
                points = new Point[]
                {
                    new Point(Convert.ToInt32(_startPosX)+5,Convert.ToInt32(_startPosY-37)),
                    new Point(Convert.ToInt32(_startPosX)+5,Convert.ToInt32(_startPosY-30)),
                    new Point(Convert.ToInt32(_startPosX+35),Convert.ToInt32(_startPosY-37)),
                };
                g.FillPolygon(dopBrush, points);
                g.DrawPolygon(pen, points);
                points = new Point[]
                {
                    new Point(Convert.ToInt32(_startPosX)+5,Convert.ToInt32(_startPosY-3)),
                    new Point(Convert.ToInt32(_startPosX)+5,Convert.ToInt32(_startPosY-10)),
                    new Point(Convert.ToInt32(_startPosX+35),Convert.ToInt32(_startPosY-3)),
                };
                g.FillPolygon(dopBrush, points);
                g.DrawPolygon(pen, points);
            }
        }
        public void SetExtraColor(Color color)
        {
            (Boat as EntityMotorBoat).DopColor = color;
        }
    }
}
