﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MotorBoat
{
    internal class DrawningObjectBoat : IDrawningObject
    {
        private DrawningBoat _boat = null;
        public DrawningObjectBoat(DrawningBoat boat)
        {
            _boat = boat;
        }
        public float Step => _boat?.Boat?.Step ?? 0;
        public (float Left, float Right, float Top, float Bottom) GetCurrentPosition()
        {
            return _boat?.GetCurrentPosition() ?? default;
        }

        public DrawningBoat GetShip => _boat;

        public void MoveObject(Direction direction)
        {
            _boat?.MoveTransport(direction);
        }
        public void SetObject(int x, int y, int width, int height)
        {
           _boat.SetPosition(x, y, width, height);
            
        }
        void IDrawningObject.DrawningObject(Graphics g)
        {
            _boat.DrawTransport(g);
        }
        public string GetInfo() => _boat?.GetDataForSave();
        public static IDrawningObject Create(string data) => new DrawningObjectBoat(data.CreateDrawningBoat());

        public bool Equals(IDrawningObject? other)
        {
            if (other == null)
            {
                return false;
            }
            var otherShip = other as DrawningObjectBoat;
            if (otherShip == null)
            {
                return false;
            }
            var ship = _boat.Boat;
            var otherShipShip = otherShip._boat.Boat;
            if (ship.GetType().Name != otherShipShip.GetType().Name)
            {
                return false;
            }
            if (ship.Speed != otherShipShip.Speed)
            {
                return false;
            }
            if (ship.Weight != otherShipShip.Weight)
            {
                return false;
            }
            if (ship.BodyColor != otherShipShip.BodyColor)
            {
                return false;
            }
            if (ship is EntityMotorBoat liner && otherShipShip is EntityMotorBoat otherLiner)
            {
                if (liner.DopColor != otherLiner.DopColor)
                {
                    return false;
                }
                if (liner.SportLine != otherLiner.SportLine)
                {
                    return false;
                }
                if (liner.Wing != otherLiner.Wing)
                {
                    return false;
                }
            }
            return true;
        }
    }
}
