﻿namespace MotorBoat
{
    /// Карта с набром объектов под нее
    internal class MapWithSetBoatsGeneric<T, U>
        where T : class, IDrawningObject, IEquatable<T>
        where U : AbstractMap
    {
        /// Ширина окна отрисовки
        private readonly int _pictureWidth;
        /// Высота окна отрисовки
        private readonly int _pictureHeight;
        /// Размер занимаемого объектом места (ширина)
        private readonly int _placeSizeWidth = 210;
        /// Размер занимаемого объектом места (высота)
        private readonly int _placeSizeHeight = 70;
        /// Набор объектов
        private readonly SetBoatsGeneric<T> _setBoats;
        /// Карта
        private readonly U _map;
        /// Конструктор
        public MapWithSetBoatsGeneric(int picWidth, int picHeight, U map)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _setBoats = new SetBoatsGeneric<T>(21);
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _map = map;
        }
        /// Перегрузка оператора сложения
        public static int operator +(MapWithSetBoatsGeneric<T, U> map, T boat)
        {
           
            return map._setBoats.Insert(boat);
        }
        /// Перегрузка оператора вычитания
        public static T operator -(MapWithSetBoatsGeneric<T, U> map, int position)
        {
            return map._setBoats.Remove(position);
        }
        /// Вывод всего набора объектов
        public Bitmap ShowSet()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawCars(gr);
            return bmp;
        }
        /// Просмотр объекта на карте
        public Bitmap ShowOnMap()
        {
            Shaking();
            foreach (var boat in _setBoats.GetBoat())
            {
                return _map.CreateMap(_pictureWidth, _pictureHeight, boat);
            }
            return new(_pictureWidth, _pictureHeight);
        }
        /// Перемещение объекта по крате
        public Bitmap MoveObject(Direction direction)
        {
            if (_map != null)
            {
                return _map.MoveObject(direction);
            }
            return new(_pictureWidth, _pictureHeight);
        }
        /// "Взбалтываем" набор, чтобы все элементы оказались в начале
        /// 

        public void Sort(IComparer<T> comparer)
        {
            _setBoats.SortSet(comparer);
        }

        private void Shaking()
        {
            int j = _setBoats.Count - 1;
            for (int i = 0; i < _setBoats.Count; i++)
            {
                if (_setBoats[i] == null)
                {
                    for (; j > i; j--)
                    {
                        var boat = _setBoats[j];
                        if (boat != null)
                        {
                            _setBoats.Insert(boat, i);
                            _setBoats.Remove(j);
                            break;
                        }
                    }
                    if (j <= i)
                    {
                        return;
                    }
                }
            }
        }
        /// Метод отрисовки фона
        private void DrawBackground(Graphics g)
        {
            g.FillRectangle(new SolidBrush(Color.LightBlue), 0, 0, _pictureWidth, _pictureHeight);
            Brush brGavan = new SolidBrush(Color.Gray);
            Brush brGreen = new SolidBrush(Color.Khaki);
            g.FillRectangle(brGreen, 0, 0, _pictureWidth, 30);
            g.FillRectangle(brGreen, _pictureWidth - 30, 0, 30, _pictureHeight);

            g.FillRectangle(brGavan, 30, 30, _pictureWidth - 60, 30);
            g.FillRectangle(brGavan, _pictureWidth - 60, 30, 30, _pictureHeight - 60);
            g.FillRectangle(brGavan, _pictureWidth / 2, 60, 60, _pictureHeight - 100);
        }
        /// Метод прорисовки объектов
        private void DrawCars(Graphics g)
        {
            int curWidth = 50;
            int curHeight = 60;
            bool right = true;
            bool down = false;
            bool rightdown = false;

            foreach (var boat in _setBoats.GetBoat())
            {
                if (right)
                {
                    boat.SetObject(curWidth, curHeight + _placeSizeHeight, _placeSizeWidth, _placeSizeHeight);
                    g.RotateTransform(270, 0);
                    g.TranslateTransform(-190, 0);
                    boat.DrawningObject(g);
                    g.TranslateTransform(190, 0);
                    g.RotateTransform(-270, 0);

                    if (curHeight > 300 && curHeight < 400)
                    {
                        curHeight += 80;
                    }
                    curHeight += _placeSizeHeight;

                    if (curHeight >= _pictureWidth - 100)
                    {
                        right = false;
                        down = true;
                        g.TranslateTransform(_pictureWidth / 2 + _placeSizeWidth, 0);
                        curWidth = 500;
                        curHeight = 200;
                    }
                }
                else if (down)
                {
                    if (rightdown)
                    {
                        rightdown = false;
                        boat.SetObject(curWidth, curHeight + _placeSizeHeight, _placeSizeWidth, _placeSizeHeight);
                        g.TranslateTransform(_placeSizeWidth + 90, 0);
                        boat.DrawningObject(g);
                        curHeight += _placeSizeHeight;
                    }
                    else
                    {
                        rightdown = true;
                        boat.SetObject(curWidth, curHeight + _placeSizeHeight, _placeSizeWidth, _placeSizeHeight);

                        g.TranslateTransform(-_placeSizeWidth - 90, 0);
                        boat.DrawningObject(g);

                    }
                    if (curHeight >= _pictureHeight - 30) { down = false; return; }

                }

            }
        }
        public string GetData(char separatorType, char separatorData)
        {
            string data = $"{_map.GetType().Name}{separatorType}";
            foreach (var boat in _setBoats.GetBoat())
            {
                data += $"{boat.GetInfo()}{separatorData}";
            }
            return data;
        }
        public void LoadData(string[] records)
        {
            foreach (var rec in records)
            {
                _setBoats.Insert(DrawningObjectBoat.Create(rec) as T);
            }
        }

    }
}