﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MotorBoat
{
    internal class MapsCollection
    {

        readonly Dictionary<string, MapWithSetBoatsGeneric<IDrawningObject, AbstractMap>> _mapStorages;
        public List<string> Keys => _mapStorages.Keys.ToList();
        private readonly int _pictureWidth;
        private readonly int _pictureHeight;
        private readonly char separatorDict = '|';
        private readonly char separatorData = ';';
        public MapsCollection(int pictureWidth, int pictureHeight)
        {
            _mapStorages = new Dictionary<string, MapWithSetBoatsGeneric<IDrawningObject, AbstractMap>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        public void AddMap(string name, AbstractMap map)
        {
            if (!_mapStorages.ContainsKey(name))
            {
                _mapStorages.Add(name, new MapWithSetBoatsGeneric<IDrawningObject, AbstractMap>(_pictureWidth, _pictureHeight, map));
            }
        }
        public void DelMap(string name)
        {
            if (_mapStorages.ContainsKey(name)) _mapStorages.Remove(name);
        }
        public MapWithSetBoatsGeneric<IDrawningObject, AbstractMap> this[string ind]
        {
            get
            {
                if (_mapStorages.ContainsKey(ind))
                {
                    return _mapStorages[ind];
                }
                return null;
            }
        }

        public void SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            using (StreamWriter fs = new StreamWriter(filename))
            {
                fs.Write($"MapsCollection{Environment.NewLine}");
                foreach (var storage in _mapStorages)
                {

                    fs.Write($"{storage.Key}{separatorDict}{storage.Value.GetData(separatorDict, separatorData)}{Environment.NewLine}");
                }
            }
        }
        public void LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new FileNotFoundException("Файл не найден");
            }
            string str = "";
            using (StreamReader fs = new StreamReader(filename))
            {
                str = fs.ReadLine();
                if (!str.Contains("MapsCollection"))
                {
                    throw new FileFormatException("Формат данных в файле не правильный");
                }
                _mapStorages.Clear();
                while ((str = fs.ReadLine()) != null)
                {
                    var elem = str.Split(separatorDict);
                    AbstractMap map = null;
                    switch (elem[1])
                    {
                        case "SimpleMap":
                            map = new SimpleMap();
                            break;
                        case "PinkMap":
                            map = new PinkMap();
                            break;
                        case "SeaMap":
                            map = new SeaMap();
                            break;
                    }
                    _mapStorages.Add(elem[0], new MapWithSetBoatsGeneric<IDrawningObject, AbstractMap>(_pictureWidth, _pictureHeight, map));
                    _mapStorages[elem[0]].LoadData(elem[2].Split(separatorData,StringSplitOptions.RemoveEmptyEntries));
                }
            }
        }

    }
}
