﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MotorBoat
{
	internal class SetBoatsGeneric<T>
		where T : class, IEquatable<T>
	{
		private readonly List<T> _places;
		public int Count => _places.Count;
		private readonly int _maxCount;
		public SetBoatsGeneric(int count)
		{
			_maxCount = count;
			_places = new List<T>();
		}
		public int Insert(T boat)
		{
            if (_places.Contains(boat))
            {
                return -1;
            }

            if (_places.Count > _maxCount)
			{
				return -1;
			}
			return Insert(boat, 0);
		}
		public int Insert(T boat, int position)
		{
			if (Count == _maxCount) { throw new StorageOverflowException(_maxCount); }
			if (position > _maxCount || position < 0) return -1;
			_places.Insert(position, boat);
			return position;
		}
		public T Remove(int position)
		{
			// TODO проверка позиции
			if (position >= Count || position < 0) throw new BoatNotFoundException(position);
			// TODO удаление объекта из массива, присовив элементу массива значение null
			T temp = _places[position];
			_places.RemoveAt(position);
			return temp;
		}
		public T this[int position]
		{
			get
			{
				if (position >= _places.Count || position < 0)
				{
					return null;
				}
				return _places[position];
			}
			set
			{
				if (position >= _places.Count || position < 0)
				{
					return;
				}
				Insert(value, position);

			}
		}
		public IEnumerable<T> GetBoat()
		{
			foreach (var boat in _places)
			{
				if (boat != null)
				{
					yield return boat;
				}
				else
				{
					yield break;
				}
			}
		}

        public void SortSet(IComparer<T> comparer)
        {
            if (comparer == null)
            {
                return;
            }
            _places.Sort(comparer);
        }
    }
}